import inspect
import typing
from .base_response import BaseResponse
import vkbottle_types.objects


class GetServerUrlResponse(BaseResponse):
    response: "GetServerUrlResponseModel" = None


class GetServerUrlResponseModel(BaseResponse):
    endpoint: typing.Optional[str] = None
    key: typing.Optional[str] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()



import inspect
import typing
from .base_response import BaseResponse
from vkbottle_types.objects import WidgetsWidgetComment, WidgetsWidgetPage


class GetCommentsResponse(BaseResponse):
    response: "GetCommentsResponseModel" = None


class GetPagesResponse(BaseResponse):
    response: "GetPagesResponseModel" = None


class GetCommentsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    posts: typing.Optional[typing.List["WidgetsWidgetComment"]] = None


class GetPagesResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    pages: typing.Optional[typing.List["WidgetsWidgetPage"]] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()



import typing
from .base_category import BaseCategory
from vkbottle_types.responses.polls import (
    AddVoteResponse,
    CreateResponse,
    DeleteVoteResponse,
    GetBackgroundsResponse,
    GetByIdResponse,
    GetVotersResponse,
    PollsBackground,
    PollsPoll,
    PollsVoters,
    SavePhotoResponse
)
from vkbottle_types.responses.base import BaseBoolInt, BaseUploadServer, GetUploadServerResponse, OkResponse


class PollsCategory(BaseCategory):
    async def add_vote(
        self,
        poll_id: int,
        answer_ids: typing.List[int],
        owner_id: typing.Optional[int] = None,
        is_board: typing.Optional[bool] = None,
        **kwargs
    ) -> BaseBoolInt:
        """Adds the current user's vote to the selected answer in the poll.
        :param poll_id: Poll ID.
        :param answer_ids:
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("polls.addVote", params)
        model = AddVoteResponse
        return model(**response).response

    async def create(
        self,
        question: typing.Optional[str] = None,
        is_anonymous: typing.Optional[bool] = None,
        is_multiple: typing.Optional[bool] = None,
        end_date: typing.Optional[int] = None,
        owner_id: typing.Optional[int] = None,
        app_id: typing.Optional[int] = None,
        add_answers: typing.Optional[str] = None,
        photo_id: typing.Optional[int] = None,
        background_id: typing.Optional[str] = None,
        disable_unvote: typing.Optional[bool] = None,
        **kwargs
    ) -> PollsPoll:
        """Creates polls that can be attached to the users' or communities' posts.
        :param question: question text
        :param is_anonymous: '1' - anonymous poll, participants list is hidden,, '0' - public poll, participants list is available,, Default value is '0'.
        :param is_multiple:
        :param end_date:
        :param owner_id: If a poll will be added to a communty it is required to send a negative group identifier. Current user by default.
        :param app_id:
        :param add_answers: available answers list, for example: " ["yes","no","maybe"]", There can be from 1 to 10 answers.
        :param photo_id:
        :param background_id:
        :param disable_unvote:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("polls.create", params)
        model = CreateResponse
        return model(**response).response

    async def delete_vote(
        self,
        poll_id: int,
        answer_id: int,
        owner_id: typing.Optional[int] = None,
        is_board: typing.Optional[bool] = None,
        **kwargs
    ) -> BaseBoolInt:
        """Deletes the current user's vote from the selected answer in the poll.
        :param poll_id: Poll ID.
        :param answer_id: Answer ID.
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("polls.deleteVote", params)
        model = DeleteVoteResponse
        return model(**response).response

    async def edit(
        self,
        poll_id: int,
        owner_id: typing.Optional[int] = None,
        question: typing.Optional[str] = None,
        add_answers: typing.Optional[str] = None,
        edit_answers: typing.Optional[str] = None,
        delete_answers: typing.Optional[str] = None,
        end_date: typing.Optional[int] = None,
        photo_id: typing.Optional[int] = None,
        background_id: typing.Optional[str] = None,
        **kwargs
    ) -> int:
        """Edits created polls
        :param poll_id: edited poll's id
        :param owner_id: poll owner id
        :param question: new question text
        :param add_answers: answers list, for example: , "["yes","no","maybe"]"
        :param edit_answers: object containing answers that need to be edited,, key - answer id, value - new answer text. Example: {"382967099":"option1", "382967103":"option2"}"
        :param delete_answers: list of answer ids to be deleted. For example: "[382967099, 382967103]"
        :param end_date:
        :param photo_id:
        :param background_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("polls.edit", params)
        model = OkResponse
        return model(**response).response

    async def get_backgrounds(
        self, **kwargs
    ) -> typing.List[PollsBackground]:
        """polls.getBackgrounds method"""

        params = self.get_set_params(locals())
        response = await self.api.request("polls.getBackgrounds", params)
        model = GetBackgroundsResponse
        return model(**response).response

    async def get_by_id(
        self,
        poll_id: int,
        owner_id: typing.Optional[int] = None,
        is_board: typing.Optional[bool] = None,
        extended: typing.Optional[bool] = None,
        friends_count: typing.Optional[int] = None,
        fields: typing.Optional[typing.List[str]] = None,
        name_case: typing.Optional[str] = None,
        **kwargs
    ) -> PollsPoll:
        """Returns detailed information about a poll by its ID.
        :param poll_id: Poll ID.
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board: '1' - poll is in a board, '0' - poll is on a wall. '0' by default.
        :param extended:
        :param friends_count:
        :param fields:
        :param name_case:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("polls.getById", params)
        model = GetByIdResponse
        return model(**response).response

    async def get_photo_upload_server(
        self, owner_id: typing.Optional[int] = None, **kwargs
    ) -> BaseUploadServer:
        """polls.getPhotoUploadServer method
        :param owner_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("polls.getPhotoUploadServer", params)
        model = GetUploadServerResponse
        return model(**response).response

    async def get_voters(
        self,
        poll_id: int,
        answer_ids: typing.List[int],
        owner_id: typing.Optional[int] = None,
        is_board: typing.Optional[bool] = None,
        friends_only: typing.Optional[bool] = None,
        offset: typing.Optional[int] = None,
        count: typing.Optional[int] = None,
        fields: typing.Optional[typing.List[str]] = None,
        name_case: typing.Optional[str] = None,
        **kwargs
    ) -> typing.List[PollsVoters]:
        """Returns a list of IDs of users who selected specific answers in the poll.
        :param poll_id: Poll ID.
        :param answer_ids: Answer IDs.
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board:
        :param friends_only: '1' — to return only current user's friends, '0' — to return all users (default),
        :param offset: Offset needed to return a specific subset of voters. '0' — (default)
        :param count: Number of user IDs to return (if the 'friends_only' parameter is not set, maximum '1000', otherwise '10'). '100' — (default)
        :param fields: Profile fields to return. Sample values: 'nickname', 'screen_name', 'sex', 'bdate (birthdate)', 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'has_mobile', 'rate', 'contacts', 'education', 'online', 'counters'.
        :param name_case: Case for declension of user name and surname: , 'nom' — nominative (default) , 'gen' — genitive , 'dat' — dative , 'acc' — accusative , 'ins' — instrumental , 'abl' — prepositional
        """

        params = self.get_set_params(locals())
        response = await self.api.request("polls.getVoters", params)
        model = GetVotersResponse
        return model(**response).response

    async def save_photo(
        self, photo: str, hash: str, **kwargs
    ) -> PollsBackground:
        """polls.savePhoto method
        :param photo:
        :param hash:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("polls.savePhoto", params)
        model = SavePhotoResponse
        return model(**response).response



import typing
from .base_category import BaseCategory
from vkbottle_types.responses.auth import RestoreResponse, RestoreResponseModel


class AuthCategory(BaseCategory):
    async def restore(
        self, phone: str, last_name: str, **kwargs
    ) -> RestoreResponseModel:
        """Allows to restore account access using a code received via SMS. " This method is only available for apps with [vk.com/dev/auth_direct|Direct authorization] access. "
        :param phone: User phone number.
        :param last_name: User last name.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("auth.restore", params)
        model = RestoreResponse
        return model(**response).response



import inspect
import typing
from .base_response import BaseResponse
from vkbottle_types.objects import (
    GroupsGroup,
    GroupsGroupFull,
    StoriesFeedItem,
    StoriesPromoBlock,
    StoriesStory,
    StoriesStoryStats,
    StoriesViewersItem,
    UsersUser,
    UsersUserFull
)


class GetBannedExtendedResponse(BaseResponse):
    response: "GetBannedExtendedResponseModel" = None


class GetBannedResponse(BaseResponse):
    response: "GetBannedResponseModel" = None


class GetByIdExtendedResponse(BaseResponse):
    response: "GetByIdExtendedResponseModel" = None


class GetByIdResponse(BaseResponse):
    response: "GetByIdResponseModel" = None


class GetPhotoUploadServerResponse(BaseResponse):
    response: "GetPhotoUploadServerResponseModel" = None


class GetStatsResponse(BaseResponse):
    response: StoriesStoryStats = None


class GetVideoUploadServerResponse(BaseResponse):
    response: "GetVideoUploadServerResponseModel" = None


class GetViewersExtendedV5115Response(BaseResponse):
    response: "GetViewersExtendedV5115ResponseModel" = None


class GetViewersExtendedResponse(BaseResponse):
    response: "GetViewersExtendedResponseModel" = None


class GetV5113Response(BaseResponse):
    response: "GetV5113ResponseModel" = None


class GetResponse(BaseResponse):
    response: "GetResponseModel" = None


class SaveResponse(BaseResponse):
    response: "SaveResponseModel" = None


class UploadResponse(BaseResponse):
    response: "UploadResponseModel" = None


class GetBannedExtendedResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List[int]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None


class GetBannedResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List[int]] = None


class GetByIdExtendedResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["StoriesStory"]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None


class GetByIdResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["StoriesStory"]] = None


class GetPhotoUploadServerResponseModel(BaseResponse):
    upload_url: typing.Optional[str] = None
    user_ids: typing.Optional[typing.List[int]] = None


class GetVideoUploadServerResponseModel(BaseResponse):
    upload_url: typing.Optional[str] = None
    user_ids: typing.Optional[typing.List[int]] = None


class GetViewersExtendedV5115ResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["StoriesViewersItem"]] = None
    hidden_reason: typing.Optional[str] = None
    next_from: typing.Optional[str] = None


class GetViewersExtendedResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["UsersUserFull"]] = None


class GetV5113ResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["StoriesFeedItem"]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroup"]] = None
    need_upload_screen: typing.Optional[bool] = None


class GetResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["list"]] = None
    promo_data: typing.Optional["StoriesPromoBlock"] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroup"]] = None
    need_upload_screen: typing.Optional[bool] = None


class SaveResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["StoriesStory"]] = None
    profiles: typing.Optional[typing.List["UsersUser"]] = None
    groups: typing.Optional[typing.List["GroupsGroup"]] = None


class UploadResponseModel(BaseResponse):
    upload_result: typing.Optional[str] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()



import inspect
import typing
from .base_response import BaseResponse
from vkbottle_types.objects import BaseSticker, StoreProduct, StoreStickersKeyword


class GetFavoriteStickersResponse(BaseResponse):
    response: typing.List["BaseSticker"] = None


class GetProductsResponse(BaseResponse):
    response: typing.List["StoreProduct"] = None


class GetStickersKeywordsResponse(BaseResponse):
    response: "GetStickersKeywordsResponseModel" = None


class GetStickersKeywordsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    dictionary: typing.Optional[typing.List["StoreStickersKeyword"]] = None
    chunks_count: typing.Optional[int] = None
    chunks_hash: typing.Optional[str] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()



import inspect
from typing import List, Optional, Union

from .base_event_object import BaseEventObject


class MessageObject(BaseEventObject):
    message_id: Optional[int] = None
    peer_id: Optional[int] = None
    timestamp: Optional[int] = None
    text: Optional[str] = None
    subject: Optional[str] = None
    info: Optional[str] = None
    attachments: Optional[Union[dict, list]] = None
    random_id: Optional[int] = None


class ReplaceMessageFlagsObject(MessageObject):
    flags: Optional[int] = None


class InstallMessageFlagsObject(MessageObject):
    mask: Optional[int] = None


class ResetMessageFlagsObject(MessageObject):
    mask: Optional[int] = None


class MessageNewObject(MessageObject):
    minor_id: Optional[int] = None
    flags: Optional[int] = None


class MessageEditObject(BaseEventObject):
    message_id: Optional[int] = None
    mask: Optional[int] = None
    peer_id: Optional[int] = None
    timestamp: Optional[int] = None
    new_text: Optional[str] = ""
    attachments: Optional[Union[dict, list]] = None


class InReadObject(BaseEventObject):
    peer_id: Optional[int] = None
    local_id: Optional[int] = None


class OutReadObject(InReadObject):
    pass


class FriendOnlineObject(BaseEventObject):
    user_id: Optional[int] = None
    extra: Optional[int] = 0
    timestamp: Optional[int] = None


class FriendOfflineObject(BaseEventObject):
    user_id: Optional[int] = None
    flags: Optional[int] = 0
    timestamp: Optional[int] = None


class ResetDialogFlagsObject(BaseEventObject):
    peer_id: Optional[int] = None
    mask: Optional[int] = None


class ReplaceDialogFlagsObject(BaseEventObject):
    peer_id: Optional[int] = None
    flags: Optional[int] = None


class InstallDialogFlagsObject(ResetDialogFlagsObject):
    pass


class MessagesDeleteObject(InReadObject):
    pass


class MessagesRestoreObject(InReadObject):
    pass


class ChangeConversationParamsObject(BaseEventObject):
    chat_id: Optional[int] = None
    self: Optional[int] = None


class DialogTypingStateObject(BaseEventObject):
    user_id: Optional[int] = None
    flags: Optional[int] = None


class ConversationTypingStateObject(BaseEventObject):
    user_id: Optional[int] = None
    chat_id: Optional[int] = None


class UsersTypingStateObject(BaseEventObject):
    user_ids: Optional[List[int]] = None
    peer_id: Optional[int] = None
    total_count: Optional[int] = None
    ts: Optional[int] = None


class CallObject(BaseEventObject):
    user_id: Optional[int] = None
    call_id: Optional[int] = None


class CounterObject(BaseEventObject):
    count: Optional[int] = None


class NotificationsSettingsChangedObject(BaseEventObject):
    peer_id: Optional[int] = None
    sound: Optional[int] = None
    disabled_until: Optional[int] = None


class ChatInfoEditObject(BaseEventObject):
    type_id: Optional[int] = None
    peer_id: Optional[int] = None
    info: Optional[Union[str, int]] = None


class ChatVoiceMessageStatesObject(BaseEventObject):
    user_ids: Optional[List[int]] = None
    peer_id: Optional[int] = None
    total_count: Optional[int] = None
    ts: Optional[int] = None


class ChatEditObject(BaseEventObject):
    chat_id: Optional[int] = None
    self: Optional[int] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseEventObject):
        item.update_forward_refs()



import inspect
import typing
from .base_response import BaseResponse
from vkbottle_types.objects import (
    GroupsGroup,
    GroupsGroupFull,
    UsersUser,
    UsersUserFull,
    WallWallComment,
    WallWallpostFull
)


class CreateCommentResponse(BaseResponse):
    response: "CreateCommentResponseModel" = None


class EditResponse(BaseResponse):
    response: "EditResponseModel" = None


class GetByIdExtendedResponse(BaseResponse):
    response: "GetByIdExtendedResponseModel" = None


class GetByIdLegacyResponse(BaseResponse):
    response: typing.List["WallWallpostFull"] = None


class GetByIdResponse(BaseResponse):
    response: "GetByIdResponseModel" = None


class GetCommentExtendedResponse(BaseResponse):
    response: "GetCommentExtendedResponseModel" = None


class GetCommentResponse(BaseResponse):
    response: "GetCommentResponseModel" = None


class GetCommentsExtendedResponse(BaseResponse):
    response: "GetCommentsExtendedResponseModel" = None


class GetCommentsResponse(BaseResponse):
    response: "GetCommentsResponseModel" = None


class GetRepostsResponse(BaseResponse):
    response: "GetRepostsResponseModel" = None


class GetExtendedResponse(BaseResponse):
    response: "GetExtendedResponseModel" = None


class GetResponse(BaseResponse):
    response: "GetResponseModel" = None


class PostAdsStealthResponse(BaseResponse):
    response: "PostAdsStealthResponseModel" = None


class PostResponse(BaseResponse):
    response: "PostResponseModel" = None


class RepostResponse(BaseResponse):
    response: "RepostResponseModel" = None


class SearchExtendedResponse(BaseResponse):
    response: "SearchExtendedResponseModel" = None


class SearchResponse(BaseResponse):
    response: "SearchResponseModel" = None


class CreateCommentResponseModel(BaseResponse):
    comment_id: typing.Optional[int] = None


class EditResponseModel(BaseResponse):
    post_id: typing.Optional[int] = None


class GetByIdExtendedResponseModel(BaseResponse):
    items: typing.Optional[typing.List["WallWallpostFull"]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None


class GetByIdResponseModel(BaseResponse):
    items: typing.Optional[typing.List["WallWallpostFull"]] = None


class GetCommentExtendedResponseModel(BaseResponse):
    items: typing.Optional[typing.List["WallWallComment"]] = None
    profiles: typing.Optional[typing.List["UsersUser"]] = None
    groups: typing.Optional[typing.List["GroupsGroup"]] = None


class GetCommentResponseModel(BaseResponse):
    items: typing.Optional[typing.List["WallWallComment"]] = None


class GetCommentsExtendedResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["WallWallComment"]] = None
    profiles: typing.Optional[typing.List["UsersUser"]] = None
    groups: typing.Optional[typing.List["GroupsGroup"]] = None
    current_level_count: typing.Optional[int] = None
    can_post: typing.Optional[bool] = None
    show_reply_button: typing.Optional[bool] = None
    groups_can_post: typing.Optional[bool] = None


class GetCommentsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["WallWallComment"]] = None
    current_level_count: typing.Optional[int] = None
    can_post: typing.Optional[bool] = None
    show_reply_button: typing.Optional[bool] = None
    groups_can_post: typing.Optional[bool] = None


class GetRepostsResponseModel(BaseResponse):
    items: typing.Optional[typing.List["WallWallpostFull"]] = None
    profiles: typing.Optional[typing.List["UsersUser"]] = None
    groups: typing.Optional[typing.List["GroupsGroup"]] = None


class GetExtendedResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["WallWallpostFull"]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None


class GetResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["WallWallpostFull"]] = None


class PostAdsStealthResponseModel(BaseResponse):
    post_id: typing.Optional[int] = None


class PostResponseModel(BaseResponse):
    post_id: typing.Optional[int] = None


class RepostResponseModel(BaseResponse):
    success: typing.Optional[int] = None
    post_id: typing.Optional[int] = None
    reposts_count: typing.Optional[int] = None
    wall_repost_count: typing.Optional[int] = None
    mail_repost_count: typing.Optional[int] = None
    likes_count: typing.Optional[int] = None


class SearchExtendedResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["WallWallpostFull"]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None


class SearchResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["WallWallpostFull"]] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()



import typing
from .base_category import BaseCategory
from vkbottle_types.responses.utils import (
    CheckLinkResponse,
    GetLastShortenedLinksResponse,
    GetLastShortenedLinksResponseModel,
    GetLinkStatsExtendedResponse,
    GetLinkStatsResponse,
    GetServerTimeResponse,
    GetShortLinkResponse,
    ResolveScreenNameResponse,
    UtilsDomainResolved,
    UtilsLinkChecked,
    UtilsLinkStats,
    UtilsShortLink
)
from vkbottle_types.responses.base import OkResponse


class UtilsCategory(BaseCategory):
    async def check_link(
        self, url: str, **kwargs
    ) -> UtilsLinkChecked:
        """Checks whether a link is blocked in VK.
        :param url: Link to check (e.g., 'http://google.com').
        """

        params = self.get_set_params(locals())
        response = await self.api.request("utils.checkLink", params)
        model = CheckLinkResponse
        return model(**response).response

    async def delete_from_last_shortened(
        self, key: str, **kwargs
    ) -> int:
        """Deletes shortened link from user's list.
        :param key: Link key (characters after vk.cc/).
        """

        params = self.get_set_params(locals())
        response = await self.api.request("utils.deleteFromLastShortened", params)
        model = OkResponse
        return model(**response).response

    async def get_last_shortened_links(
        self,
        count: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        **kwargs
    ) -> GetLastShortenedLinksResponseModel:
        """Returns a list of user's shortened links.
        :param count: Number of links to return.
        :param offset: Offset needed to return a specific subset of links.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("utils.getLastShortenedLinks", params)
        model = GetLastShortenedLinksResponse
        return model(**response).response

    async def get_link_stats(
        self,
        key: str,
        source: typing.Optional[str] = None,
        access_key: typing.Optional[str] = None,
        interval: typing.Optional[str] = None,
        intervals_count: typing.Optional[int] = None,
        extended: typing.Optional[bool] = None,
        **kwargs
    ) -> UtilsLinkStats:
        """Returns stats data for shortened link.
        :param key: Link key (characters after vk.cc/).
        :param source: Source of scope
        :param access_key: Access key for private link stats.
        :param interval: Interval.
        :param intervals_count: Number of intervals to return.
        :param extended: 1 — to return extended stats data (sex, age, geo). 0 — to return views number only.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("utils.getLinkStats", params)
        model = self.get_model(
            {("extended",): GetLinkStatsExtendedResponse},
            default=GetLinkStatsResponse,
            params=params,
        )
        return model(**response).response

    async def get_server_time(
        self, **kwargs
    ) -> int:
        """Returns the current time of the VK server."""

        params = self.get_set_params(locals())
        response = await self.api.request("utils.getServerTime", params)
        model = GetServerTimeResponse
        return model(**response).response

    async def get_short_link(
        self, url: str, private: typing.Optional[bool] = None, **kwargs
    ) -> UtilsShortLink:
        """Allows to receive a link shortened via vk.cc.
        :param url: URL to be shortened.
        :param private: 1 — private stats, 0 — public stats.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("utils.getShortLink", params)
        model = GetShortLinkResponse
        return model(**response).response

    async def resolve_screen_name(
        self, screen_name: str, **kwargs
    ) -> UtilsDomainResolved:
        """Detects a type of object (e.g., user, community, application) and its ID by screen name.
        :param screen_name: Screen name of the user, community (e.g., 'apiclub,' 'andrew', or 'rules_of_war'), or application.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("utils.resolveScreenName", params)
        model = ResolveScreenNameResponse
        return model(**response).response



import typing
from .base_category import BaseCategory
from vkbottle_types.responses.database import (
    BaseCountry,
    BaseObject,
    DatabaseStation,
    GetChairsResponse,
    GetChairsResponseModel,
    GetCitiesByIdResponse,
    GetCitiesResponse,
    GetCitiesResponseModel,
    GetCountriesByIdResponse,
    GetCountriesResponse,
    GetCountriesResponseModel,
    GetFacultiesResponse,
    GetFacultiesResponseModel,
    GetMetroStationsByIdResponse,
    GetMetroStationsResponse,
    GetMetroStationsResponseModel,
    GetRegionsResponse,
    GetRegionsResponseModel,
    GetSchoolClassesResponse,
    GetSchoolsResponse,
    GetSchoolsResponseModel,
    GetUniversitiesResponse,
    GetUniversitiesResponseModel
)


class DatabaseCategory(BaseCategory):
    async def get_chairs(
        self,
        faculty_id: int,
        offset: typing.Optional[int] = None,
        count: typing.Optional[int] = None,
        **kwargs
    ) -> GetChairsResponseModel:
        """Returns list of chairs on a specified faculty.
        :param faculty_id: id of the faculty to get chairs from
        :param offset: offset required to get a certain subset of chairs
        :param count: amount of chairs to get
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getChairs", params)
        model = GetChairsResponse
        return model(**response).response

    async def get_cities(
        self,
        country_id: int,
        region_id: typing.Optional[int] = None,
        q: typing.Optional[str] = None,
        need_all: typing.Optional[bool] = None,
        offset: typing.Optional[int] = None,
        count: typing.Optional[int] = None,
        **kwargs
    ) -> GetCitiesResponseModel:
        """Returns a list of cities.
        :param country_id: Country ID.
        :param region_id: Region ID.
        :param q: Search query.
        :param need_all: '1' — to return all cities in the country, '0' — to return major cities in the country (default),
        :param offset: Offset needed to return a specific subset of cities.
        :param count: Number of cities to return.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getCities", params)
        model = GetCitiesResponse
        return model(**response).response

    async def get_cities_by_id(
        self, city_ids: typing.Optional[typing.List[int]] = None, **kwargs
    ) -> typing.List[BaseObject]:
        """Returns information about cities by their IDs.
        :param city_ids: City IDs.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getCitiesById", params)
        model = GetCitiesByIdResponse
        return model(**response).response

    async def get_countries(
        self,
        need_all: typing.Optional[bool] = None,
        code: typing.Optional[str] = None,
        offset: typing.Optional[int] = None,
        count: typing.Optional[int] = None,
        **kwargs
    ) -> GetCountriesResponseModel:
        """Returns a list of countries.
        :param need_all: '1' — to return a full list of all countries, '0' — to return a list of countries near the current user's country (default).
        :param code: Country codes in [vk.com/dev/country_codes|ISO 3166-1 alpha-2] standard.
        :param offset: Offset needed to return a specific subset of countries.
        :param count: Number of countries to return.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getCountries", params)
        model = GetCountriesResponse
        return model(**response).response

    async def get_countries_by_id(
        self, country_ids: typing.Optional[typing.List[int]] = None, **kwargs
    ) -> typing.List[BaseCountry]:
        """Returns information about countries by their IDs.
        :param country_ids: Country IDs.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getCountriesById", params)
        model = GetCountriesByIdResponse
        return model(**response).response

    async def get_faculties(
        self,
        university_id: int,
        offset: typing.Optional[int] = None,
        count: typing.Optional[int] = None,
        **kwargs
    ) -> GetFacultiesResponseModel:
        """Returns a list of faculties (i.e., university departments).
        :param university_id: University ID.
        :param offset: Offset needed to return a specific subset of faculties.
        :param count: Number of faculties to return.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getFaculties", params)
        model = GetFacultiesResponse
        return model(**response).response

    async def get_metro_stations(
        self,
        city_id: int,
        offset: typing.Optional[int] = None,
        count: typing.Optional[int] = None,
        extended: typing.Optional[bool] = None,
        **kwargs
    ) -> GetMetroStationsResponseModel:
        """Get metro stations by city
        :param city_id:
        :param offset:
        :param count:
        :param extended:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getMetroStations", params)
        model = GetMetroStationsResponse
        return model(**response).response

    async def get_metro_stations_by_id(
        self, station_ids: typing.Optional[typing.List[int]] = None, **kwargs
    ) -> typing.List[DatabaseStation]:
        """Get metro station by his id
        :param station_ids:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getMetroStationsById", params)
        model = GetMetroStationsByIdResponse
        return model(**response).response

    async def get_regions(
        self,
        country_id: int,
        q: typing.Optional[str] = None,
        offset: typing.Optional[int] = None,
        count: typing.Optional[int] = None,
        **kwargs
    ) -> GetRegionsResponseModel:
        """Returns a list of regions.
        :param country_id: Country ID, received in [vk.com/dev/database.getCountries|database.getCountries] method.
        :param q: Search query.
        :param offset: Offset needed to return specific subset of regions.
        :param count: Number of regions to return.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getRegions", params)
        model = GetRegionsResponse
        return model(**response).response

    async def get_school_classes(
        self, country_id: typing.Optional[int] = None, **kwargs
    ) -> typing.List[list]:
        """Returns a list of school classes specified for the country.
        :param country_id: Country ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getSchoolClasses", params)
        model = GetSchoolClassesResponse
        return model(**response).response

    async def get_schools(
        self,
        city_id: int,
        q: typing.Optional[str] = None,
        offset: typing.Optional[int] = None,
        count: typing.Optional[int] = None,
        **kwargs
    ) -> GetSchoolsResponseModel:
        """Returns a list of schools.
        :param city_id: City ID.
        :param q: Search query.
        :param offset: Offset needed to return a specific subset of schools.
        :param count: Number of schools to return.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getSchools", params)
        model = GetSchoolsResponse
        return model(**response).response

    async def get_universities(
        self,
        q: typing.Optional[str] = None,
        country_id: typing.Optional[int] = None,
        city_id: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        count: typing.Optional[int] = None,
        **kwargs
    ) -> GetUniversitiesResponseModel:
        """Returns a list of higher education institutions.
        :param q: Search query.
        :param country_id: Country ID.
        :param city_id: City ID.
        :param offset: Offset needed to return a specific subset of universities.
        :param count: Number of universities to return.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("database.getUniversities", params)
        model = GetUniversitiesResponse
        return model(**response).response



import inspect
import typing
from .base_response import BaseResponse
from vkbottle_types.objects import (
    GroupsGroupFull,
    NewsfeedList,
    NewsfeedListFull,
    NewsfeedNewsfeedItem,
    UsersSubscriptionsItem,
    UsersUserFull,
    WallWallpostFull,
    WallWallpostToId
)


class GetBannedExtendedResponse(BaseResponse):
    response: "GetBannedExtendedResponseModel" = None


class GetBannedResponse(BaseResponse):
    response: "GetBannedResponseModel" = None


class GetCommentsResponse(BaseResponse):
    response: "GetCommentsResponseModel" = None


class GetListsExtendedResponse(BaseResponse):
    response: "GetListsExtendedResponseModel" = None


class GetListsResponse(BaseResponse):
    response: "GetListsResponseModel" = None


class GetMentionsResponse(BaseResponse):
    response: "GetMentionsResponseModel" = None


class GetRecommendedResponse(BaseResponse):
    response: "GetRecommendedResponseModel" = None


class GetSuggestedSourcesResponse(BaseResponse):
    response: "GetSuggestedSourcesResponseModel" = None


class GetResponse(BaseResponse):
    response: "GetResponseModel" = None


class IgnoreItemResponse(BaseResponse):
    response: "IgnoreItemResponseModel" = None


class SaveListResponse(BaseResponse):
    response: int = None


class SearchExtendedResponse(BaseResponse):
    response: "SearchExtendedResponseModel" = None


class SearchResponse(BaseResponse):
    response: "SearchResponseModel" = None


class GetBannedExtendedResponseModel(BaseResponse):
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None


class GetBannedResponseModel(BaseResponse):
    groups: typing.Optional[typing.List[int]] = None
    members: typing.Optional[typing.List[int]] = None


class GetCommentsResponseModel(BaseResponse):
    items: typing.Optional[typing.List["NewsfeedNewsfeedItem"]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None
    next_from: typing.Optional[str] = None


class GetListsExtendedResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["NewsfeedListFull"]] = None


class GetListsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["NewsfeedList"]] = None


class GetMentionsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["WallWallpostToId"]] = None


class GetRecommendedResponseModel(BaseResponse):
    items: typing.Optional[typing.List["NewsfeedNewsfeedItem"]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None
    next_from: typing.Optional[str] = None


class GetSuggestedSourcesResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["UsersSubscriptionsItem"]] = None


class GetResponseModel(BaseResponse):
    items: typing.Optional[typing.List["NewsfeedNewsfeedItem"]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None
    next_from: typing.Optional[str] = None


class IgnoreItemResponseModel(BaseResponse):
    status: typing.Optional[bool] = None


class SearchExtendedResponseModel(BaseResponse):
    items: typing.Optional[typing.List["WallWallpostFull"]] = None
    profiles: typing.Optional[typing.List["UsersUserFull"]] = None
    groups: typing.Optional[typing.List["GroupsGroupFull"]] = None
    suggested_queries: typing.Optional[typing.List[str]] = None
    next_from: typing.Optional[str] = None
    count: typing.Optional[int] = None
    total_count: typing.Optional[int] = None


class SearchResponseModel(BaseResponse):
    items: typing.Optional[typing.List["WallWallpostFull"]] = None
    suggested_queries: typing.Optional[typing.List[str]] = None
    next_from: typing.Optional[str] = None
    count: typing.Optional[int] = None
    total_count: typing.Optional[int] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()



import inspect
import typing
from .base_response import BaseResponse
from vkbottle_types.objects import (
    BaseCountry,
    BaseObject,
    DatabaseCity,
    DatabaseFaculty,
    DatabaseRegion,
    DatabaseSchool,
    DatabaseStation,
    DatabaseUniversity
)


class GetChairsResponse(BaseResponse):
    response: "GetChairsResponseModel" = None


class GetCitiesByIdResponse(BaseResponse):
    response: typing.List["BaseObject"] = None


class GetCitiesResponse(BaseResponse):
    response: "GetCitiesResponseModel" = None


class GetCountriesByIdResponse(BaseResponse):
    response: typing.List["BaseCountry"] = None


class GetCountriesResponse(BaseResponse):
    response: "GetCountriesResponseModel" = None


class GetFacultiesResponse(BaseResponse):
    response: "GetFacultiesResponseModel" = None


class GetMetroStationsByIdResponse(BaseResponse):
    response: typing.List["DatabaseStation"] = None


class GetMetroStationsResponse(BaseResponse):
    response: "GetMetroStationsResponseModel" = None


class GetRegionsResponse(BaseResponse):
    response: "GetRegionsResponseModel" = None


class GetSchoolClassesResponse(BaseResponse):
    response: typing.List[list] = None


class GetSchoolsResponse(BaseResponse):
    response: "GetSchoolsResponseModel" = None


class GetUniversitiesResponse(BaseResponse):
    response: "GetUniversitiesResponseModel" = None


class GetChairsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["BaseObject"]] = None


class GetCitiesResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["DatabaseCity"]] = None


class GetCountriesResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["BaseCountry"]] = None


class GetFacultiesResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["DatabaseFaculty"]] = None


class GetMetroStationsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["DatabaseStation"]] = None


class GetRegionsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["DatabaseRegion"]] = None


class GetSchoolsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["DatabaseSchool"]] = None


class GetUniversitiesResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["DatabaseUniversity"]] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()



import typing
from .base_category import BaseCategory
from vkbottle_types.responses.leadForms import (
    CreateResponse,
    CreateResponseModel,
    DeleteResponse,
    DeleteResponseModel,
    GetLeadsResponse,
    GetLeadsResponseModel,
    GetResponse,
    LeadFormsForm,
    ListResponse,
    UploadUrlResponse
)


class LeadFormsCategory(BaseCategory):
    async def create(
        self,
        group_id: int,
        name: str,
        title: str,
        description: str,
        questions: str,
        policy_link_url: str,
        photo: typing.Optional[str] = None,
        confirmation: typing.Optional[str] = None,
        site_link_url: typing.Optional[str] = None,
        active: typing.Optional[bool] = None,
        once_per_user: typing.Optional[bool] = None,
        pixel_code: typing.Optional[str] = None,
        notify_admins: typing.Optional[typing.List[int]] = None,
        notify_emails: typing.Optional[typing.List[str]] = None,
        **kwargs
    ) -> CreateResponseModel:
        """leadForms.create method
        :param group_id:
        :param name:
        :param title:
        :param description:
        :param questions:
        :param policy_link_url:
        :param photo:
        :param confirmation:
        :param site_link_url:
        :param active:
        :param once_per_user:
        :param pixel_code:
        :param notify_admins:
        :param notify_emails:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.create", params)
        model = CreateResponse
        return model(**response).response

    async def delete(
        self, group_id: int, form_id: int, **kwargs
    ) -> DeleteResponseModel:
        """leadForms.delete method
        :param group_id:
        :param form_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.delete", params)
        model = DeleteResponse
        return model(**response).response

    async def get(
        self, group_id: int, form_id: int, **kwargs
    ) -> LeadFormsForm:
        """leadForms.get method
        :param group_id:
        :param form_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.get", params)
        model = GetResponse
        return model(**response).response

    async def get_leads(
        self,
        group_id: int,
        form_id: int,
        limit: typing.Optional[int] = None,
        next_page_token: typing.Optional[str] = None,
        **kwargs
    ) -> GetLeadsResponseModel:
        """leadForms.getLeads method
        :param group_id:
        :param form_id:
        :param limit:
        :param next_page_token:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.getLeads", params)
        model = GetLeadsResponse
        return model(**response).response

    async def get_upload_u_r_l(
        self, **kwargs
    ) -> str:
        """leadForms.getUploadURL method"""

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.getUploadURL", params)
        model = UploadUrlResponse
        return model(**response).response

    async def list(
        self, group_id: int, **kwargs
    ) -> typing.List[LeadFormsForm]:
        """leadForms.list method
        :param group_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.list", params)
        model = ListResponse
        return model(**response).response

    async def update(
        self,
        group_id: int,
        form_id: int,
        name: str,
        title: str,
        description: str,
        questions: str,
        policy_link_url: str,
        photo: typing.Optional[str] = None,
        confirmation: typing.Optional[str] = None,
        site_link_url: typing.Optional[str] = None,
        active: typing.Optional[bool] = None,
        once_per_user: typing.Optional[bool] = None,
        pixel_code: typing.Optional[str] = None,
        notify_admins: typing.Optional[typing.List[int]] = None,
        notify_emails: typing.Optional[typing.List[str]] = None,
        **kwargs
    ) -> CreateResponseModel:
        """leadForms.update method
        :param group_id:
        :param form_id:
        :param name:
        :param title:
        :param description:
        :param questions:
        :param policy_link_url:
        :param photo:
        :param confirmation:
        :param site_link_url:
        :param active:
        :param once_per_user:
        :param pixel_code:
        :param notify_admins:
        :param notify_emails:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("leadForms.update", params)
        model = CreateResponse
        return model(**response).response



import inspect
import typing
from .base_response import BaseResponse
from vkbottle_types.objects import BaseBoolInt, PollsBackground, PollsPoll, PollsVoters


class AddVoteResponse(BaseResponse):
    response: BaseBoolInt = None


class CreateResponse(BaseResponse):
    response: PollsPoll = None


class DeleteVoteResponse(BaseResponse):
    response: BaseBoolInt = None


class GetBackgroundsResponse(BaseResponse):
    response: typing.List["PollsBackground"] = None


class GetByIdResponse(BaseResponse):
    response: PollsPoll = None


class GetVotersResponse(BaseResponse):
    response: typing.List["PollsVoters"] = None


class SavePhotoResponse(BaseResponse):
    response: PollsBackground = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()



import typing
from .base_category import BaseCategory
from vkbottle_types.responses.downloadedGames import PaidStatusResponse, PaidStatusResponseModel


class DownloadedGamesCategory(BaseCategory):
    async def get_paid_status(
        self, user_id: typing.Optional[int] = None, **kwargs
    ) -> PaidStatusResponseModel:
        """downloadedGames.getPaidStatus method
        :param user_id:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("downloadedGames.getPaidStatus", params)
        model = PaidStatusResponse
        return model(**response).response



import typing
from .base_category import BaseCategory
from vkbottle_types.responses.ads import (
    AddOfficeUsersResponse,
    AdsAccount,
    AdsAd,
    AdsAdLayout,
    AdsCampaign,
    AdsClient,
    AdsDemoStats,
    AdsFloodStats,
    AdsLinkStatus,
    AdsPromotedPostReach,
    AdsRejectReason,
    AdsStats,
    AdsTargSettings,
    AdsTargStats,
    AdsTargSuggestions,
    AdsTargetGroup,
    AdsUpdateOfficeUsersResult,
    AdsUsers,
    CheckLinkResponse,
    CreateAdsResponse,
    CreateCampaignsResponse,
    CreateClientsResponse,
    CreateTargetGroupResponse,
    CreateTargetGroupResponseModel,
    DeleteAdsResponse,
    DeleteCampaignsResponse,
    DeleteClientsResponse,
    GetAccountsResponse,
    GetAdsLayoutResponse,
    GetAdsResponse,
    GetAdsTargetingResponse,
    GetBudgetResponse,
    GetCampaignsResponse,
    GetCategoriesResponse,
    GetCategoriesResponseModel,
    GetClientsResponse,
    GetDemographicsResponse,
    GetFloodStatsResponse,
    GetLookalikeRequestsResponse,
    GetLookalikeRequestsResponseModel,
    GetMusiciansResponse,
    GetMusiciansResponseModel,
    GetOfficeUsersResponse,
    GetPostsReachResponse,
    GetRejectionReasonResponse,
    GetStatisticsResponse,
    GetSuggestionsCitiesResponse,
    GetSuggestionsRegionsResponse,
    GetSuggestionsResponse,
    GetSuggestionsSchoolsResponse,
    GetTargetGroupsResponse,
    GetTargetingStatsResponse,
    GetUploadURLResponse,
    GetVideoUploadURLResponse,
    ImportTargetContactsResponse,
    RemoveOfficeUsersResponse,
    UpdateAdsResponse,
    UpdateCampaignsResponse,
    UpdateClientsResponse,
    UpdateOfficeUsersResponse
)
from vkbottle_types.responses.base import OkResponse


class AdsCategory(BaseCategory):
    async def add_office_users(
        self, account_id: int, data: str, **kwargs
    ) -> bool:
        """Adds managers and/or supervisors to advertising account.
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe added managers. Description of 'user_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.addOfficeUsers", params)
        model = AddOfficeUsersResponse
        return model(**response).response

    async def check_link(
        self,
        account_id: int,
        link_type: str,
        link_url: str,
        campaign_id: typing.Optional[int] = None,
        **kwargs
    ) -> AdsLinkStatus:
        """Allows to check the ad link.
        :param account_id: Advertising account ID.
        :param link_type: Object type: *'community' — community,, *'post' — community post,, *'application' — VK application,, *'video' — video,, *'site' — external site.
        :param link_url: Object URL.
        :param campaign_id: Campaign ID
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.checkLink", params)
        model = CheckLinkResponse
        return model(**response).response

    async def create_ads(
        self, account_id: int, data: str, **kwargs
    ) -> typing.List[int]:
        """Creates ads.
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created ads. Description of 'ad_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createAds", params)
        model = CreateAdsResponse
        return model(**response).response

    async def create_campaigns(
        self, account_id: int, data: str, **kwargs
    ) -> typing.List[int]:
        """Creates advertising campaigns.
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created campaigns. Description of 'campaign_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createCampaigns", params)
        model = CreateCampaignsResponse
        return model(**response).response

    async def create_clients(
        self, account_id: int, data: str, **kwargs
    ) -> typing.List[int]:
        """Creates clients of an advertising agency.
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created campaigns. Description of 'client_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createClients", params)
        model = CreateClientsResponse
        return model(**response).response

    async def create_target_group(
        self,
        account_id: int,
        name: str,
        lifetime: int,
        client_id: typing.Optional[int] = None,
        target_pixel_id: typing.Optional[int] = None,
        target_pixel_rules: typing.Optional[str] = None,
        **kwargs
    ) -> CreateTargetGroupResponseModel:
        """Creates a group to re-target ads for users who visited advertiser's site (viewed information about the product, registered, etc.).
        :param account_id: Advertising account ID.
        :param name: Name of the target group — a string up to 64 characters long.
        :param lifetime: 'For groups with auditory created with pixel code only.', , Number of days after that users will be automatically removed from the group.
        :param client_id: 'Only for advertising agencies.', ID of the client with the advertising account where the group will be created.
        :param target_pixel_id:
        :param target_pixel_rules:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.createTargetGroup", params)
        model = CreateTargetGroupResponse
        return model(**response).response

    async def delete_ads(
        self, account_id: int, ids: str, **kwargs
    ) -> typing.List[int]:
        """Archives ads.
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with ad IDs.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.deleteAds", params)
        model = DeleteAdsResponse
        return model(**response).response

    async def delete_campaigns(
        self, account_id: int, ids: str, **kwargs
    ) -> int:
        """Archives advertising campaigns.
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted campaigns.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.deleteCampaigns", params)
        model = DeleteCampaignsResponse
        return model(**response).response

    async def delete_clients(
        self, account_id: int, ids: str, **kwargs
    ) -> int:
        """Archives clients of an advertising agency.
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted clients.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.deleteClients", params)
        model = DeleteClientsResponse
        return model(**response).response

    async def delete_target_group(
        self,
        account_id: int,
        target_group_id: int,
        client_id: typing.Optional[int] = None,
        **kwargs
    ) -> int:
        """Deletes a retarget group.
        :param account_id: Advertising account ID.
        :param target_group_id: Group ID.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.deleteTargetGroup", params)
        model = OkResponse
        return model(**response).response

    async def get_accounts(
        self, **kwargs
    ) -> typing.List[AdsAccount]:
        """Returns a list of advertising accounts."""

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getAccounts", params)
        model = GetAccountsResponse
        return model(**response).response

    async def get_ads(
        self,
        account_id: int,
        ad_ids: typing.Optional[str] = None,
        campaign_ids: typing.Optional[str] = None,
        client_id: typing.Optional[int] = None,
        include_deleted: typing.Optional[bool] = None,
        only_deleted: typing.Optional[bool] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        **kwargs
    ) -> typing.List[AdsAd]:
        """Returns number of ads.
        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'Available and required for advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown: *0 — show only active ads,, *1 — show all ads.
        :param only_deleted: Flag that specifies whether to show only archived ads: *0 — show all ads,, *1 — show only archived ads. Available when include_deleted flag is *1
        :param limit: Limit of number of returned ads. Used only if ad_ids parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset. Used in the same cases as 'limit' parameter.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getAds", params)
        model = GetAdsResponse
        return model(**response).response

    async def get_ads_layout(
        self,
        account_id: int,
        client_id: typing.Optional[int] = None,
        include_deleted: typing.Optional[bool] = None,
        only_deleted: typing.Optional[bool] = None,
        campaign_ids: typing.Optional[str] = None,
        ad_ids: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        **kwargs
    ) -> typing.List[AdsAdLayout]:
        """Returns descriptions of ad layouts.
        :param account_id: Advertising account ID.
        :param client_id: 'For advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown. *0 — show only active ads,, *1 — show all ads.
        :param only_deleted: Flag that specifies whether to show only archived ads: *0 — show all ads,, *1 — show only archived ads. Available when include_deleted flag is *1
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param limit: Limit of number of returned ads. Used only if 'ad_ids' parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset. Used in the same cases as 'limit' parameter.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getAdsLayout", params)
        model = GetAdsLayoutResponse
        return model(**response).response

    async def get_ads_targeting(
        self,
        account_id: int,
        ad_ids: typing.Optional[str] = None,
        campaign_ids: typing.Optional[str] = None,
        client_id: typing.Optional[int] = None,
        include_deleted: typing.Optional[bool] = None,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        **kwargs
    ) -> typing.List[AdsTargSettings]:
        """Returns ad targeting parameters.
        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'For advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: flag that specifies whether archived ads shall be shown: *0 — show only active ads,, *1 — show all ads.
        :param limit: Limit of number of returned ads. Used only if 'ad_ids' parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset needed to return a specific subset of results.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getAdsTargeting", params)
        model = GetAdsTargetingResponse
        return model(**response).response

    async def get_budget(
        self, account_id: int, **kwargs
    ) -> int:
        """Returns current budget of the advertising account.
        :param account_id: Advertising account ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getBudget", params)
        model = GetBudgetResponse
        return model(**response).response

    async def get_campaigns(
        self,
        account_id: int,
        client_id: typing.Optional[int] = None,
        include_deleted: typing.Optional[bool] = None,
        campaign_ids: typing.Optional[str] = None,
        fields: typing.Optional[typing.List[str]] = None,
        **kwargs
    ) -> typing.List[AdsCampaign]:
        """Returns a list of campaigns in an advertising account.
        :param account_id: Advertising account ID.
        :param client_id: 'For advertising agencies'. ID of the client advertising campaigns are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown. *0 — show only active campaigns,, *1 — show all campaigns.
        :param campaign_ids: Filter of advertising campaigns to show. Serialized JSON array with campaign IDs. Only campaigns that exist in 'campaign_ids' and belong to the specified advertising account will be shown. If the parameter is null, all campaigns will be shown.
        :param fields:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getCampaigns", params)
        model = GetCampaignsResponse
        return model(**response).response

    async def get_categories(
        self, lang: typing.Optional[str] = None, **kwargs
    ) -> GetCategoriesResponseModel:
        """Returns a list of possible ad categories.
        :param lang: Language. The full list of supported languages is [vk.com/dev/api_requests|here].
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getCategories", params)
        model = GetCategoriesResponse
        return model(**response).response

    async def get_clients(
        self, account_id: int, **kwargs
    ) -> typing.List[AdsClient]:
        """Returns a list of advertising agency's clients.
        :param account_id: Advertising account ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getClients", params)
        model = GetClientsResponse
        return model(**response).response

    async def get_demographics(
        self,
        account_id: int,
        ids_type: str,
        ids: str,
        period: str,
        date_from: str,
        date_to: str,
        **kwargs
    ) -> typing.List[AdsDemoStats]:
        """Returns demographics for ads or campaigns.
        :param account_id: Advertising account ID.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad — ads,, *campaign — campaigns.
        :param ids: IDs requested ads or campaigns, separated with a comma, depending on the value set in 'ids_type'. Maximum 2000 objects.
        :param period: Data grouping by dates: *day — statistics by days,, *month — statistics by months,, *overall — overall statistics. 'date_from' and 'date_to' parameters set temporary limits.
        :param date_from: Date to show statistics from. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — day it was created on,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — month it was created in,, *overall: 0.
        :param date_to: Date to show statistics to. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — current day,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — current month,, *overall: 0.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getDemographics", params)
        model = GetDemographicsResponse
        return model(**response).response

    async def get_flood_stats(
        self, account_id: int, **kwargs
    ) -> AdsFloodStats:
        """Returns information about current state of a counter — number of remaining runs of methods and time to the next counter nulling in seconds.
        :param account_id: Advertising account ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getFloodStats", params)
        model = GetFloodStatsResponse
        return model(**response).response

    async def get_lookalike_requests(
        self,
        account_id: int,
        client_id: typing.Optional[int] = None,
        requests_ids: typing.Optional[str] = None,
        offset: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        sort_by: typing.Optional[str] = None,
        **kwargs
    ) -> GetLookalikeRequestsResponseModel:
        """ads.getLookalikeRequests method
        :param account_id:
        :param client_id:
        :param requests_ids:
        :param offset:
        :param limit:
        :param sort_by:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getLookalikeRequests", params)
        model = GetLookalikeRequestsResponse
        return model(**response).response

    async def get_musicians(
        self, artist_name: str, **kwargs
    ) -> GetMusiciansResponseModel:
        """ads.getMusicians method
        :param artist_name:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getMusicians", params)
        model = GetMusiciansResponse
        return model(**response).response

    async def get_musicians_by_ids(
        self, ids: typing.List[int], **kwargs
    ) -> GetMusiciansResponseModel:
        """ads.getMusiciansByIds method
        :param ids:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getMusiciansByIds", params)
        model = GetMusiciansResponse
        return model(**response).response

    async def get_office_users(
        self, account_id: int, **kwargs
    ) -> typing.List[AdsUsers]:
        """Returns a list of managers and supervisors of advertising account.
        :param account_id: Advertising account ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getOfficeUsers", params)
        model = GetOfficeUsersResponse
        return model(**response).response

    async def get_posts_reach(
        self, account_id: int, ids_type: str, ids: str, **kwargs
    ) -> typing.List[AdsPromotedPostReach]:
        """Returns detailed statistics of promoted posts reach from campaigns and ads.
        :param account_id: Advertising account ID.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad — ads,, *campaign — campaigns.
        :param ids: IDs requested ads or campaigns, separated with a comma, depending on the value set in 'ids_type'. Maximum 100 objects.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getPostsReach", params)
        model = GetPostsReachResponse
        return model(**response).response

    async def get_rejection_reason(
        self, account_id: int, ad_id: int, **kwargs
    ) -> AdsRejectReason:
        """Returns a reason of ad rejection for pre-moderation.
        :param account_id: Advertising account ID.
        :param ad_id: Ad ID.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getRejectionReason", params)
        model = GetRejectionReasonResponse
        return model(**response).response

    async def get_statistics(
        self,
        account_id: int,
        ids_type: str,
        ids: str,
        period: str,
        date_from: str,
        date_to: str,
        stats_fields: typing.Optional[typing.List[str]] = None,
        **kwargs
    ) -> typing.List[AdsStats]:
        """Returns statistics of performance indicators for ads, campaigns, clients or the whole account.
        :param account_id: Advertising account ID.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad — ads,, *campaign — campaigns,, *client — clients,, *office — account.
        :param ids: IDs requested ads, campaigns, clients or account, separated with a comma, depending on the value set in 'ids_type'. Maximum 2000 objects.
        :param period: Data grouping by dates: *day — statistics by days,, *month — statistics by months,, *overall — overall statistics. 'date_from' and 'date_to' parameters set temporary limits.
        :param date_from: Date to show statistics from. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — day it was created on,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — month it was created in,, *overall: 0.
        :param date_to: Date to show statistics to. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — current day,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — current month,, *overall: 0.
        :param stats_fields: Additional fields to add to statistics
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getStatistics", params)
        model = GetStatisticsResponse
        return model(**response).response

    async def get_suggestions(
        self,
        section: str,
        ids: typing.Optional[str] = None,
        q: typing.Optional[str] = None,
        country: typing.Optional[int] = None,
        cities: typing.Optional[str] = None,
        lang: typing.Optional[str] = None,
        **kwargs
    ) -> typing.List[AdsTargSuggestions]:
        """Returns a set of auto-suggestions for various targeting parameters.
        :param section: Section, suggestions are retrieved in. Available values: *countries — request of a list of countries. If q is not set or blank, a short list of countries is shown. Otherwise, a full list of countries is shown. *regions — requested list of regions. 'country' parameter is required. *cities — requested list of cities. 'country' parameter is required. *districts — requested list of districts. 'cities' parameter is required. *stations — requested list of subway stations. 'cities' parameter is required. *streets — requested list of streets. 'cities' parameter is required. *schools — requested list of educational organizations. 'cities' parameter is required. *interests — requested list of interests. *positions — requested list of positions (professions). *group_types — requested list of group types. *religions — requested list of religious commitments. *browsers — requested list of browsers and mobile devices.
        :param ids: Objects IDs separated by commas. If the parameter is passed, 'q, country, cities' should not be passed.
        :param q: Filter-line of the request (for countries, regions, cities, streets, schools, interests, positions).
        :param country: ID of the country objects are searched in.
        :param cities: IDs of cities where objects are searched in, separated with a comma.
        :param lang: Language of the returned string values. Supported languages: *ru — Russian,, *ua — Ukrainian,, *en — English.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getSuggestions", params)
        model = self.get_model(
            {
                ("regions",): GetSuggestionsRegionsResponse,
                ("cities",): GetSuggestionsCitiesResponse,
                ("schools",): GetSuggestionsSchoolsResponse,
            },
            default=GetSuggestionsResponse,
            params=params,
        )
        return model(**response).response

    async def get_target_groups(
        self,
        account_id: int,
        client_id: typing.Optional[int] = None,
        extended: typing.Optional[bool] = None,
        **kwargs
    ) -> typing.List[AdsTargetGroup]:
        """Returns a list of target groups.
        :param account_id: Advertising account ID.
        :param client_id: 'Only for advertising agencies.', ID of the client with the advertising account where the group will be created.
        :param extended: '1' — to return pixel code.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getTargetGroups", params)
        model = GetTargetGroupsResponse
        return model(**response).response

    async def get_targeting_stats(
        self,
        account_id: int,
        link_url: str,
        client_id: typing.Optional[int] = None,
        criteria: typing.Optional[str] = None,
        ad_id: typing.Optional[int] = None,
        ad_format: typing.Optional[int] = None,
        ad_platform: typing.Optional[str] = None,
        ad_platform_no_wall: typing.Optional[str] = None,
        ad_platform_no_ad_network: typing.Optional[str] = None,
        publisher_platforms: typing.Optional[str] = None,
        link_domain: typing.Optional[str] = None,
        need_precise: typing.Optional[bool] = None,
        impressions_limit_period: typing.Optional[int] = None,
        **kwargs
    ) -> AdsTargStats:
        """Returns the size of targeting audience, and also recommended values for CPC and CPM.
        :param account_id: Advertising account ID.
        :param link_url: URL for the advertised object.
        :param client_id:
        :param criteria: Serialized JSON object that describes targeting parameters. Description of 'criteria' object see below.
        :param ad_id: ID of an ad which targeting parameters shall be analyzed.
        :param ad_format: Ad format. Possible values: *'1' — image and text,, *'2' — big image,, *'3' — exclusive format,, *'4' — community, square image,, *'7' — special app format,, *'8' — special community format,, *'9' — post in community,, *'10' — app board.
        :param ad_platform: Platforms to use for ad showing. Possible values: (for 'ad_format' = '1'), *'0' — VK and partner sites,, *'1' — VK only. (for 'ad_format' = '9'), *'all' — all platforms,, *'desktop' — desktop version,, *'mobile' — mobile version and apps.
        :param ad_platform_no_wall:
        :param ad_platform_no_ad_network:
        :param publisher_platforms:
        :param link_domain: Domain of the advertised object.
        :param need_precise: Additionally return recommended cpc and cpm to reach 5,10..95 percents of audience.
        :param impressions_limit_period: Impressions limit period in seconds, must be a multiple of 86400(day)
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getTargetingStats", params)
        model = GetTargetingStatsResponse
        return model(**response).response

    async def get_upload_u_r_l(
        self, ad_format: int, icon: typing.Optional[int] = None, **kwargs
    ) -> str:
        """Returns URL to upload an ad photo to.
        :param ad_format: Ad format: *1 — image and text,, *2 — big image,, *3 — exclusive format,, *4 — community, square image,, *7 — special app format.
        :param icon:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getUploadURL", params)
        model = GetUploadURLResponse
        return model(**response).response

    async def get_video_upload_u_r_l(
        self, **kwargs
    ) -> str:
        """Returns URL to upload an ad video to."""

        params = self.get_set_params(locals())
        response = await self.api.request("ads.getVideoUploadURL", params)
        model = GetVideoUploadURLResponse
        return model(**response).response

    async def import_target_contacts(
        self,
        account_id: int,
        target_group_id: int,
        contacts: str,
        client_id: typing.Optional[int] = None,
        **kwargs
    ) -> int:
        """Imports a list of advertiser's contacts to count VK registered users against the target group.
        :param account_id: Advertising account ID.
        :param target_group_id: Target group ID.
        :param contacts: List of phone numbers, emails or user IDs separated with a comma.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.importTargetContacts", params)
        model = ImportTargetContactsResponse
        return model(**response).response

    async def remove_office_users(
        self, account_id: int, ids: str, **kwargs
    ) -> bool:
        """Removes managers and/or supervisors from advertising account.
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted managers.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.removeOfficeUsers", params)
        model = RemoveOfficeUsersResponse
        return model(**response).response

    async def update_ads(
        self, account_id: int, data: str, **kwargs
    ) -> typing.List[int]:
        """Edits ads.
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in ads. Description of 'ad_edit_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateAds", params)
        model = UpdateAdsResponse
        return model(**response).response

    async def update_campaigns(
        self, account_id: int, data: str, **kwargs
    ) -> int:
        """Edits advertising campaigns.
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in campaigns. Description of 'campaign_mod' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateCampaigns", params)
        model = UpdateCampaignsResponse
        return model(**response).response

    async def update_clients(
        self, account_id: int, data: str, **kwargs
    ) -> int:
        """Edits clients of an advertising agency.
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in clients. Description of 'client_mod' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateClients", params)
        model = UpdateClientsResponse
        return model(**response).response

    async def update_office_users(
        self, account_id: int, data: str, **kwargs
    ) -> typing.List[AdsUpdateOfficeUsersResult]:
        """Adds managers and/or supervisors to advertising account.
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe added managers. Description of 'user_specification' objects see below.
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateOfficeUsers", params)
        model = UpdateOfficeUsersResponse
        return model(**response).response

    async def update_target_group(
        self,
        account_id: int,
        target_group_id: int,
        name: str,
        lifetime: int,
        client_id: typing.Optional[int] = None,
        domain: typing.Optional[str] = None,
        target_pixel_id: typing.Optional[int] = None,
        target_pixel_rules: typing.Optional[str] = None,
        **kwargs
    ) -> int:
        """Edits a retarget group.
        :param account_id: Advertising account ID.
        :param target_group_id: Group ID.
        :param name: New name of the target group — a string up to 64 characters long.
        :param lifetime: 'Only for the groups that get audience from sites with user accounting code.', Time in days when users added to a retarget group will be automatically excluded from it. '0' - automatic exclusion is off.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        :param domain: Domain of the site where user accounting code will be placed.
        :param target_pixel_id:
        :param target_pixel_rules:
        """

        params = self.get_set_params(locals())
        response = await self.api.request("ads.updateTargetGroup", params)
        model = OkResponse
        return model(**response).response



import inspect
import typing
from .base_response import BaseResponse
from vkbottle_types.objects import LeadFormsForm, LeadFormsLead


class CreateResponse(BaseResponse):
    response: "CreateResponseModel" = None


class DeleteResponse(BaseResponse):
    response: "DeleteResponseModel" = None


class GetLeadsResponse(BaseResponse):
    response: "GetLeadsResponseModel" = None


class GetResponse(BaseResponse):
    response: LeadFormsForm = None


class ListResponse(BaseResponse):
    response: typing.List["LeadFormsForm"] = None


class UploadUrlResponse(BaseResponse):
    response: str = None


class CreateResponseModel(BaseResponse):
    form_id: typing.Optional[int] = None
    url: typing.Optional[str] = None


class DeleteResponseModel(BaseResponse):
    form_id: typing.Optional[int] = None


class GetLeadsResponseModel(BaseResponse):
    leads: typing.Optional[typing.List["LeadFormsLead"]] = None
    next_page_token: typing.Optional[str] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()



import inspect
import typing
from .base_response import BaseResponse
from vkbottle_types.objects import (
    AdsAccount,
    AdsAd,
    AdsAdLayout,
    AdsCampaign,
    AdsCategory,
    AdsClient,
    AdsDemoStats,
    AdsFloodStats,
    AdsLinkStatus,
    AdsLookalikeRequest,
    AdsMusician,
    AdsPromotedPostReach,
    AdsRejectReason,
    AdsStats,
    AdsTargSettings,
    AdsTargStats,
    AdsTargSuggestions,
    AdsTargSuggestionsCities,
    AdsTargSuggestionsRegions,
    AdsTargSuggestionsSchools,
    AdsTargetGroup,
    AdsUpdateOfficeUsersResult,
    AdsUsers
)


class AddOfficeUsersResponse(BaseResponse):
    response: bool = None


class CheckLinkResponse(BaseResponse):
    response: AdsLinkStatus = None


class CreateAdsResponse(BaseResponse):
    response: typing.List[int] = None


class CreateCampaignsResponse(BaseResponse):
    response: typing.List[int] = None


class CreateClientsResponse(BaseResponse):
    response: typing.List[int] = None


class CreateTargetGroupResponse(BaseResponse):
    response: "CreateTargetGroupResponseModel" = None


class DeleteAdsResponse(BaseResponse):
    response: typing.List[int] = None


class DeleteCampaignsResponse(BaseResponse):
    response: int = None


class DeleteClientsResponse(BaseResponse):
    response: int = None


class GetAccountsResponse(BaseResponse):
    response: typing.List["AdsAccount"] = None


class GetAdsLayoutResponse(BaseResponse):
    response: typing.List["AdsAdLayout"] = None


class GetAdsTargetingResponse(BaseResponse):
    response: typing.List["AdsTargSettings"] = None


class GetAdsResponse(BaseResponse):
    response: typing.List["AdsAd"] = None


class GetBudgetResponse(BaseResponse):
    response: int = None


class GetCampaignsResponse(BaseResponse):
    response: typing.List["AdsCampaign"] = None


class GetCategoriesResponse(BaseResponse):
    response: "GetCategoriesResponseModel" = None


class GetClientsResponse(BaseResponse):
    response: typing.List["AdsClient"] = None


class GetDemographicsResponse(BaseResponse):
    response: typing.List["AdsDemoStats"] = None


class GetFloodStatsResponse(BaseResponse):
    response: AdsFloodStats = None


class GetLookalikeRequestsResponse(BaseResponse):
    response: "GetLookalikeRequestsResponseModel" = None


class GetMusiciansResponse(BaseResponse):
    response: "GetMusiciansResponseModel" = None


class GetOfficeUsersResponse(BaseResponse):
    response: typing.List["AdsUsers"] = None


class GetPostsReachResponse(BaseResponse):
    response: typing.List["AdsPromotedPostReach"] = None


class GetRejectionReasonResponse(BaseResponse):
    response: AdsRejectReason = None


class GetStatisticsResponse(BaseResponse):
    response: typing.List["AdsStats"] = None


class GetSuggestionsCitiesResponse(BaseResponse):
    response: typing.List["AdsTargSuggestionsCities"] = None


class GetSuggestionsRegionsResponse(BaseResponse):
    response: typing.List["AdsTargSuggestionsRegions"] = None


class GetSuggestionsResponse(BaseResponse):
    response: typing.List["AdsTargSuggestions"] = None


class GetSuggestionsSchoolsResponse(BaseResponse):
    response: typing.List["AdsTargSuggestionsSchools"] = None


class GetTargetGroupsResponse(BaseResponse):
    response: typing.List["AdsTargetGroup"] = None


class GetTargetingStatsResponse(BaseResponse):
    response: AdsTargStats = None


class GetUploadURLResponse(BaseResponse):
    response: str = None


class GetVideoUploadURLResponse(BaseResponse):
    response: str = None


class ImportTargetContactsResponse(BaseResponse):
    response: int = None


class RemoveOfficeUsersResponse(BaseResponse):
    response: bool = None


class UpdateAdsResponse(BaseResponse):
    response: typing.List[int] = None


class UpdateCampaignsResponse(BaseResponse):
    response: int = None


class UpdateClientsResponse(BaseResponse):
    response: int = None


class UpdateOfficeUsersResponse(BaseResponse):
    response: typing.List["AdsUpdateOfficeUsersResult"] = None


class CreateTargetGroupResponseModel(BaseResponse):
    id: typing.Optional[int] = None
    pixel: typing.Optional[str] = None


class GetCategoriesResponseModel(BaseResponse):
    v1: typing.Optional[typing.List["AdsCategory"]] = None
    v2: typing.Optional[typing.List["AdsCategory"]] = None


class GetLookalikeRequestsResponseModel(BaseResponse):
    count: typing.Optional[int] = None
    items: typing.Optional[typing.List["AdsLookalikeRequest"]] = None


class GetMusiciansResponseModel(BaseResponse):
    items: typing.Optional[typing.List["AdsMusician"]] = None


for item in locals().copy().values():
    if inspect.isclass(item) and issubclass(item, BaseResponse):
        item.update_forward_refs()
